@@ -33,7 +33,6 @@ func (p *InstallControllers) Prepare(config *v1beta1.Cluster) error {
 	p.hosts = p.Config.Spec.Hosts.Controllers().Filter(func(h *cluster.Host) bool {
 		return !h.Reset && !h.Metadata.NeedsUpgrade && (h != p.leader && h.Metadata.K0sRunningVersion == nil)
 	})
-
 	return nil
 }
 
@@ -65,13 +64,13 @@ func (p *InstallControllers) CleanUp() {
 
 func (p *InstallControllers) After() error {
 	for i, h := range p.hosts {
-		if h.Metadata.K0sJoinTokenID == "" {
+		if h.Metadata.K0sTokenData.Token == "" {
 			continue
 		}
-		h.Metadata.K0sJoinToken = ""
+		h.Metadata.K0sTokenData.Token = ""
 		err := p.Wet(p.leader, fmt.Sprintf("invalidate k0s join token for controller %s", h), func() error {
 			log.Debugf("%s: invalidating join token for controller %d", p.leader, i+1)
-			return p.leader.Exec(p.leader.Configurer.K0sCmdf("token invalidate --data-dir=%s %s", p.leader.K0sDataDir(), h.Metadata.K0sJoinTokenID), exec.Sudo(p.leader))
+			return p.leader.Exec(p.leader.Configurer.K0sCmdf("token invalidate --data-dir=%s %s", p.leader.K0sDataDir(), h.Metadata.K0sTokenData.ID), exec.Sudo(p.leader))
 		})
 		if err != nil {
 			log.Warnf("%s: failed to invalidate worker join token: %v", p.leader, err)
@@ -88,33 +87,10 @@ func (p *InstallControllers) After() error {
 
 // Run the phase
 func (p *InstallControllers) Run() error {
-	url := p.Config.Spec.InternalKubeAPIURL()
-	healthz := fmt.Sprintf("%s/healthz", url)
-
-	err := p.parallelDo(p.hosts, func(h *cluster.Host) error {
-		if p.IsWet() || !p.leader.Metadata.DryRunFakeLeader {
-			log.Infof("%s: validating api connection to %s", h, url)
-			ctx, cancel := context.WithTimeout(context.Background(), 30*time.Second)
-			defer cancel()
-			if err := retry.Context(ctx, node.HTTPStatusFunc(h, healthz, 200, 401)); err != nil {
-				return fmt.Errorf("failed to connect from controller to kubernetes api at %s - check networking", url)
-			}
-		} else {
-			log.Warnf("%s: dry-run: skipping api connection validation to %s because cluster is not running", h, url)
-		}
-		return nil
-	})
-	if err != nil {
-		return err
-	}
-
 	for _, h := range p.hosts {
-		var token string
-		var tokenID string
-
 		if p.IsWet() {
-			log.Infof("%s: generating token", p.leader)
-			token, err = p.Config.Spec.K0s.GenerateToken(
+			log.Infof("%s: generate join token for %s", p.leader, h)
+			token, err := p.Config.Spec.K0s.GenerateToken(
 				p.leader,
 				"controller",
 				time.Duration(10)*time.Minute,
@@ -122,20 +98,40 @@ func (p *InstallControllers) Run() error {
 			if err != nil {
 				return err
 			}
-			h.Metadata.K0sJoinToken = token
-			tokenID, err = cluster.TokenID(token)
+			tokenData, err := cluster.ParseToken(token)
 			if err != nil {
 				return err
 			}
-			log.Debugf("%s: join token ID: %s", p.leader, tokenID)
-			h.Metadata.K0sJoinTokenID = tokenID
+			h.Metadata.K0sTokenData = tokenData
 		} else {
 			p.DryMsgf(p.leader, "generate a k0s join token for controller %s", h)
-			h.Metadata.K0sJoinTokenID = "dry-run"
+			h.Metadata.K0sTokenData.ID = "dry-run"
+			h.Metadata.K0sTokenData.URL = p.Config.Spec.KubeAPIURL()
 		}
-
-		log.Infof("%s: writing join token", h)
-		if err := h.Configurer.WriteFile(h, h.K0sJoinTokenPath(), h.Metadata.K0sJoinToken, "0640"); err != nil {
+	}
+	err := p.parallelDo(p.hosts, func(h *cluster.Host) error {
+		if p.IsWet() || !p.leader.Metadata.DryRunFakeLeader {
+			log.Infof("%s: validating api connection to %s", h, h.Metadata.K0sTokenData.URL)
+			ctx, cancel := context.WithTimeout(context.Background(), 30*time.Second)
+			defer cancel()
+			if err := retry.Context(ctx, node.HTTPStatusFunc(h, h.Metadata.K0sTokenData.URL, 200, 401, 404)); err != nil {
+				return fmt.Errorf("failed to connect from controller to kubernetes api - check networking: %w", err)
+			}
+		} else {
+			log.Warnf("%s: dry-run: skipping api connection validation to because cluster is not actually running", h)
+		}
+		return nil
+	})
+	if err != nil {
+		return err
+	}
+	return p.parallelDo(p.hosts, func(h *cluster.Host) error {
+		tokenPath := h.K0sJoinTokenPath()
+		log.Infof("%s: writing join token to %s", h, tokenPath)
+		err := p.Wet(h, fmt.Sprintf("write k0s join token to %s", tokenPath), func() error {
+			return h.Configurer.WriteFile(h, tokenPath, h.Metadata.K0sTokenData.Token, "0600")
+		})
+		if err != nil {
 			return err
 		}
 
@@ -180,17 +176,22 @@ func (p *InstallControllers) Run() error {
 				return err
 			}
 
-			if err := p.waitJoined(h); err != nil {
-				return err
+			ctx, cancel := context.WithTimeout(context.Background(), 30*time.Second)
+			defer cancel()
+			err := retry.Context(ctx, func(_ context.Context) error {
+				out, err := h.ExecOutput(h.Configurer.KubectlCmdf(h, h.K0sDataDir(), "get --raw='/readyz?verbose=true'"), exec.Sudo(h))
+				if err != nil {
+					return fmt.Errorf("readiness endpoint reports %q: %w", out, err)
+				}
+				return nil
+			})
+			if err != nil {
+				return fmt.Errorf("controller did not reach ready state: %w", err)
 			}
+
+			h.Metadata.Ready = true
 		}
-		h.Metadata.Ready = true
-	}
 
-	return nil
-}
-
-func (p *InstallControllers) waitJoined(h *cluster.Host) error {
-	log.Infof("%s: waiting for kubernetes api to respond", h)
-	return retry.Timeout(context.TODO(), retry.DefaultTimeout, node.KubeAPIReadyFunc(h, p.Config))
+		return nil
+	})
 }
