@@ -6,7 +6,7 @@ import (
 
 	"github.com/creasty/defaults"
 	"github.com/jellydator/validation"
-	"github.com/k0sproject/dig"
+	"gopkg.in/yaml.v2"
 )
 
 // Spec defines cluster config spec section
@@ -82,22 +82,48 @@ func (s *Spec) Validate() error {
 	)
 }
 
-func (s *Spec) clusterExternalAddress() string {
-	if a := s.K0s.Config.DigString("spec", "api", "externalAddress"); a != "" {
-		return a
-	}
+type k0sCPLBConfig struct {
+	Spec struct {
+		Network struct {
+			ControlPlaneLoadBalancing struct {
+				Enabled    bool   `yaml:"enabled"`
+				Type       string `yaml:"type"`
+				Keepalived struct {
+					VirtualServers []struct {
+						IPAddress string `yaml:"ipAddress"`
+					} `yaml:"virtualServers"`
+				} `yaml:"keepalived"`
+			} `yaml:"controlPlaneLoadBalancing"`
+		} `yaml:"network"`
+	} `yaml:"spec"`
+}
 
-	if cplb, ok := s.K0s.Config.Dig("spec", "network", "controlPlaneLoadBalancing").(dig.Mapping); ok {
-		if enabled, ok := cplb.Dig("enabled").(bool); ok && enabled && cplb.DigString("type") == "Keepalived" {
-			if vrrpAddresses, ok := cplb.Dig("keepalived", "virtualServers").([]dig.Mapping); ok && len(vrrpAddresses) > 0 {
-				if addr, ok := vrrpAddresses[0]["ipAddress"].(string); ok && addr != "" {
-					return addr
+func (s *Spec) clusterExternalAddress() string {
+	if s.K0s != nil {
+		if a := s.K0s.Config.DigString("spec", "api", "externalAddress"); a != "" {
+			return a
+		}
+
+		if cfg, err := yaml.Marshal(s.K0s.Config); err == nil {
+			k0scfg := k0sCPLBConfig{}
+			if err := yaml.Unmarshal(cfg, &k0scfg); err == nil {
+				cplb := k0scfg.Spec.Network.ControlPlaneLoadBalancing
+				if cplb.Enabled && cplb.Type == "Keepalived" {
+					for _, vs := range cplb.Keepalived.VirtualServers {
+						if addr := vs.IPAddress; addr != "" {
+							return addr
+						}
+					}
 				}
 			}
 		}
 	}
 
-	return s.K0sLeader().Address()
+	if leader := s.K0sLeader(); leader != nil {
+		return leader.Address()
+	}
+
+	return ""
 }
 
 func (s *Spec) clusterInternalAddress() string {
@@ -112,8 +138,10 @@ func (s *Spec) clusterInternalAddress() string {
 const defaultAPIPort = 6443
 
 func (s *Spec) APIPort() int {
-	if p, ok := s.K0s.Config.Dig("spec", "api", "port").(int); ok {
-		return p
+	if s.K0s != nil {
+		if p, ok := s.K0s.Config.Dig("spec", "api", "port").(int); ok {
+			return p
+		}
 	}
 	return defaultAPIPort
 }
